/***********************************************************************************/
/**	\file				RNPacket.cpp
**	\brief				Implementation zur Klasse RNPacket
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		29.03.2002
**	letzte Aenderung:	01.04.2002
*************************************************************************************
**	Die Klasse RNPacket sendet oder empfngt Daten in der Form von beliebigen
**  Datenblcken.
************************************************************************************/


// includes
#include "RNPacket.h"

// Methoden-Definitionen

/************************************************************************************
**	Methode:			RNPacket::RNPacket
************************************************************************************/
/**	Konstruktor
***
************************************************************************************/
RNPacket::RNPacket()
{
	m_bInitialized=false;
	m_pmcCallBack=NULL;
	m_pvoidUserContext=NULL;

	m_iBufferUsage=0;
}

/************************************************************************************
**	Methode:			RNPacket::~RNPacket
************************************************************************************/
/**	Destruktor
***
************************************************************************************/
RNPacket::~RNPacket()
{
	socketClose();
}

/************************************************************************************
**	Methode:			RNPacket::initMessageCallBack
************************************************************************************/
/**	Initialisiert den MessageHandler Callback, bevor dies nicht geschehen ist, 
*** knnen keine Aufrufe zu diesem Objekt Erfolg haben. Der UserContext wird bei 
***	Aufruf der CallBack-Funktion bergeben, dies ist z.B. ntzlich wenn mehrere 
***	RNPackets zu den gleichen MessageHandler aufrufen. (kann aber auch einfach NULL 
***	sein)
************************************************************************************/
RNPRESULT RNPacket::initMessageCallBack(	PVOID pvoidUserContext,
											const PMESSAGECALLBACK pmcFoo)
{
	if (pmcFoo == NULL)
	{
		return RNP_ERROR_INVALID_PARAM;
	}

	m_pmcCallBack=pmcFoo;

	m_pvoidUserContext=pvoidUserContext;

	m_bInitialized=true;

	return RNP_OK;
}

/************************************************************************************
**	Methode:			RNPacket::recvMessage
************************************************************************************/
/**	Versucht Daten vom angegebenen Socket zu empfangen, wenn ein Packet empfangen
*** werden kann, wird dieses der MessageCallBack bergeben.
************************************************************************************/
RNPRESULT RNPacket::recvMessage(void)
{
	if (m_bInitialized == false)
	{
		return RNP_ERROR_NOT_INITIALIZED;
	}

	bool bReadable;

	if (!m_RNSSocket.isReadable(bReadable,0))
	{
		return RNP_ERROR_SOCKET;
	}

	if (!bReadable)
	{
		return RNP_SOCKET_NOT_READABLE;
	}

	WORD wLength;

	if (m_iBufferUsage<c_iMaxPacketLength)
	{
		int iBytesReceived=c_iMaxPacketLength-m_iBufferUsage;

		if(!m_RNSSocket.receive(	m_pcBuffer+m_iBufferUsage,
									&iBytesReceived))
		{
			return RNP_SOCKET_NOT_CONNECTED;
		}

		m_iBufferUsage=+iBytesReceived;

		if (iBytesReceived==0)
		{
			return RNP_SOCKET_NOT_CONNECTED;
		}
	}

	while (true)
	{
		if (m_iBufferUsage<2)
		{
			// noch nichtmal die 2 Header Bytes sind da...
			return RNP_OK;
		}
		
		wLength=*(WORD*)m_pcBuffer;
		
		if (m_iBufferUsage<2+wLength)
		{
			// kein komplettes Paket da
			return RNP_OK;
		}
		
		// Speicher fr gesamtes Paket besorgen und abholen
		void* pvoidData= new char[wLength];
		
		// Speicher holen fehlgeschlagen?!
		if (pvoidData==NULL)
		{
			return RNP_ERROR_UNDEFINED; // sollte _nie_ vorkommen
		}
		
		// Paket in den Speicher kopieren
		memcpy(pvoidData,(void*)((char*)m_pcBuffer+2),wLength);
		
		// Bufferlnge anpassem
		m_iBufferUsage-=(wLength+2);
		
		if (m_iBufferUsage>0)
		{
			// ein Teil des nchsten Pakets ist schon da
			memmove((void*)m_pcBuffer,(void*)(m_pcBuffer+(wLength+2)),m_iBufferUsage);
		}
		
		// den Buffer an den MessageHandler schicken
		RNPRESULT rnpTmp;
		
		rnpTmp=m_pmcCallBack(m_pvoidUserContext,pvoidData, wLength);
		
		if (rnpTmp == RNP_BUFFER_RETURNED)
		{
			delete pvoidData;
		}
	}

	//	return RNP_OK;
}

/************************************************************************************
**	Methode:			RNPacket::sendMessage
************************************************************************************/
/**	Verschickt ein Paket mit der entsprechenden Grsse ber den Socket. Wird nicht
*** alles gesendet ist davon auszugehen, das die Verbindung unterbrochen ist, da ein
***	Paket kaputt ist.
************************************************************************************/
RNPRESULT RNPacket::sendMessage(void* pvoidData, WORD wBufferSize)
{
	if (pvoidData == NULL)
	{
		return RNP_ERROR_INVALID_PARAM;
	}
	
	if (wBufferSize == 0)
	{
		return RNP_ERROR_INVALID_PARAM;
	}
	
	if (m_bInitialized == false)
	{
		return RNP_ERROR_NOT_INITIALIZED;
	}

	bool bWriteable;

	if (!m_RNSSocket.isWriteable(bWriteable,0))
	{
		return RNP_ERROR_SOCKET;
	}

	if (!bWriteable)
	{
		return RNP_SOCKET_NOT_WRITEABLE;
	}

	//Packet und Lngeninfo zusammenpacken
	int iLength=2+wBufferSize; 

	char* pcBuffer= new char[iLength];

	*((WORD*)pcBuffer)=wBufferSize;

	memcpy((void*)(pcBuffer+2),pvoidData,wBufferSize);

	if (!m_RNSSocket.send(pcBuffer, &iLength))
	{
		return RNP_ERROR_SOCKET;
	}

	return RNP_OK;
}

/************************************************************************************
**	Methode:			RNPacket::socketConnect
************************************************************************************/
/**	Versucht eine Verbindung zur angegebenen Adresse aufzubauen.
*** 
************************************************************************************/
RNPRESULT RNPacket::socketConnect(	char* pcDestName, 
									int iPort ,
									int iAdressFamily /* = AF_INET  */, 
									int iType /* = SOCK_STREAM */, 
									int iProtocol /* = IPPROTO_IP */	)
{
	if (m_bInitialized == false)
	{
		return RNP_ERROR_NOT_INITIALIZED;
	}

	if(!m_RNSSocket.create(iAdressFamily, iType, iProtocol))
	{
		return RNP_ERROR_SOCKET;
	}

	if(!m_RNSSocket.connect(pcDestName, iPort, iAdressFamily))
	{
		return RNP_SOCKET_NOT_CONNECTED;
	}

	return RNP_OK;
}

/************************************************************************************
**	Methode:			RNPacket::socketAccept
************************************************************************************/
/** Die Methode akzeptiert eine eingehende Verbinding, die am Listener registriert
*** wird.
************************************************************************************/
RNPRESULT RNPacket::socketAccept(RNSocket* psocketListener)
{
	if (m_bInitialized == false)
	{
		return RNP_ERROR_NOT_INITIALIZED;
	}
	
	bool bReadable;

	if(!psocketListener->isReadable(bReadable,100))
	{
		return RNP_ERROR_SOCKET;
	}

	if (!bReadable)
	{
		return RNP_SOCKET_NOT_CONNECTED;
	}

	if(!psocketListener->accept(m_RNSSocket))
	{
		return RNP_SOCKET_NOT_CONNECTED;
	}

	return RNP_OK;
}

/************************************************************************************
**	Methode:			RNPacket::socketClose
************************************************************************************/
/**	Schliesst den Socket.
***
************************************************************************************/
RNPRESULT RNPacket::socketClose(void)
{
	if (m_bInitialized == false)
	{
		return RNP_ERROR_NOT_INITIALIZED;
	}

	if(!m_RNSSocket.close())
	{
		return RNP_ERROR_SOCKET;
	}

	return RNP_OK;
}

/************************************************************************************
**	Ende der Datei:		RNPacket.cpp
************************************************************************************/
