/***********************************************************************************/
/**	\file				RNSocket.cpp
**	\brief				Implementation zur Klasse RNSocket
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		27.03.2002
**	letzte Aenderung:	31.03.2002
*************************************************************************************
**	Die Klasse RNSocket beitet eine Klasse zur Kapselung von WinSock.
************************************************************************************/


// includes
#include "RNSocket.h"

bool	RNSocket::m_bSocketInitialized = false;
WSADATA RNSocket::m_wsaData;

// Methoden-Definitionen

/************************************************************************************
**	Methode:			RNSocket::RNSocket
************************************************************************************/
/**	Konstruktor
************************************************************************************/
RNSocket::RNSocket()
{
	m_Socket				= INVALID_SOCKET;

	ZeroMemory(&m_addrLocal, sizeof(m_addrLocal));
	ZeroMemory(&m_addrTarget, sizeof(m_addrTarget));
}

/************************************************************************************
**	Methode:			RNSocket::~RNSocket
************************************************************************************/
/**	Destruktor
************************************************************************************/
RNSocket::~RNSocket()
{
	close();
}

/************************************************************************************
**	Methode:			RNSocket::init
************************************************************************************/
/**	Initialisiert die Sockets mit der angegebenen Vesion. (Ldt die Windows dll)
************************************************************************************/
bool RNSocket::init(int iMajorVersion /* = 1 */, int iMinorVersion /* = 1 */)
{
	//Version erzeugen
	WORD wVersionReg = MAKEWORD((WORD)iMajorVersion, (WORD)iMinorVersion);

	if(m_bSocketInitialized)
	{
		// schon initialisiert?	
		return false;
	}

	if (WSAStartup(wVersionReg, &m_wsaData) != 0)
	{
		m_bSocketInitialized=false;
		return false;
	}

	if (	LOBYTE( m_wsaData.wVersion ) != iMajorVersion ||
			HIBYTE( m_wsaData.wVersion ) != iMinorVersion )
	{
		WSACleanup();
		m_bSocketInitialized=false;
		return false; 
	}
 
	m_bSocketInitialized=true;
	return true;
}

/************************************************************************************
**	Methode:			RNSocket::cleanUp
************************************************************************************/
/**	Entldt die Winsock dll mit Hilfe von WSACleanup.
************************************************************************************/
bool RNSocket::cleanUp(void)
{
	HRESULT hResult;

	// sind die Sockets ueberhaupt noch initialisiert ?!
	if(!m_bSocketInitialized)
	{
		return false;
	}

	hResult = WSACleanup();

	if (hResult == SOCKET_ERROR)
	{
		return false;
	}

	m_bSocketInitialized = false;

	// erfolgreich aufgerumt!
	return true;
}

/************************************************************************************
**	Methode:			RNSocket::create
************************************************************************************/
/**	Erstellt den Socket mit den entsprechenden Parametern.
************************************************************************************/
bool RNSocket::create(int iAdressFamily/* =AF_INET */,
					  int iType/* =SOCK_STREAM */,
					  int iProtocol/* =IPPROTO_IP */)
{
	if (m_Socket != INVALID_SOCKET)
	{
		close();
	}
		
	m_Socket = socket(	iAdressFamily,
						iType,
						iProtocol);

	if (m_Socket == INVALID_SOCKET)
	{
		return false;
    }

	//wird spter fr bind gebraucht
	m_addrLocal.sin_family=iAdressFamily;

	
	BOOL bYes=TRUE;

	if ( setsockopt(	m_Socket, 
						SOL_SOCKET, 
						SO_REUSEADDR, 
						(char*)&bYes, 
						sizeof(bYes)) == SOCKET_ERROR) 
	{
		return false;
	}
	

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::close
************************************************************************************/
/**	Schliesst den Socket.
************************************************************************************/
bool RNSocket::close()
{
	int iResult;

	iResult = closesocket(m_Socket);

	if(iResult == SOCKET_ERROR)
	{
		return false;
	}

	m_Socket=INVALID_SOCKET;

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::shutdown
************************************************************************************/
/**	Ruft shutdown() fr den Socket mit den gewnschten Parametern auf.
************************************************************************************/
bool RNSocket::shutdown(int iFlags)
{
	int iResult;

	iResult = ::shutdown(m_Socket, iFlags);
	
	if(iResult == SOCKET_ERROR)
	{
		return false;
	}

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::connect
************************************************************************************/
/**	ffnet eine Verbindung zur angegebenen IP-Adresse und Port.
************************************************************************************/
bool RNSocket::connect(char* pcDestName, int iPort, int iAddressFamily/* =AF_INET */)
{
	int iResult;

	// Ziel bestimmen
	m_addrTarget.sin_family		= iAddressFamily;
	m_addrTarget.sin_port			= htons((WORD)iPort);
	m_addrTarget.sin_addr.s_addr	= inet_addr(pcDestName);
		
	// und verbinden
	iResult = ::connect(m_Socket, 
						(struct sockaddr *) &m_addrTarget,
						sizeof(m_addrTarget));

	if (iResult == SOCKET_ERROR)
	{
		return false;
	}

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::bind
************************************************************************************/
/**	Bindet den Socket an den angegebenem Namen (IP) und Port.
************************************************************************************/
bool RNSocket::bind(char* pcDestName, int iPort)
{
	int iResult;

	// Spezification unseres Socket bestimmen
	if(pcDestName == NULL)
	{
		m_addrLocal.sin_addr.s_addr = htonl(INADDR_ANY);
	}
	else
	{
		m_addrLocal.sin_addr.s_addr = inet_addr(pcDestName);
	}

	m_addrLocal.sin_port = htons((WORD)iPort);


	iResult = ::bind(	m_Socket, 
						(struct sockaddr*) &m_addrLocal, 
						sizeof(m_addrLocal));
	
	if(iResult == SOCKET_ERROR)
	{
		return false;
	}

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::listen
************************************************************************************/
/**	Veranlasst das Lauschen an einem Socket, der vorher ge"bind"et wurde.
*** iBackLog gibt an wie Connection Requests gequeued werden.
************************************************************************************/
bool RNSocket::listen(int iBackLog/* =SOMAXCONN */)
{
	int iResult;

	iResult = ::listen(m_Socket, iBackLog);

	if( iResult == SOCKET_ERROR)
	{
		return false;
	}

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::accept
************************************************************************************/
/**	Akzeptiert eine Verbindung, ab sofort knnen ber den neuen Socket Daten
*** ausgetauscht werden.
************************************************************************************/
bool RNSocket::accept(RNSocket &socketNew)
{
	int iAddrTargetLength = sizeof(socketNew.m_addrTarget);

	// anfragende Verbindung akzeptieren
	socketNew.m_Socket = ::accept(	m_Socket, 
									(struct sockaddr*) &socketNew.m_addrTarget, 
									&iAddrTargetLength);

	if(socketNew.m_Socket == INVALID_SOCKET)
	{
		return false;
	}

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::receive
************************************************************************************/
/**	Empfngt Daten auf dem Socket, es werden maximal piLength Daten empfangen. Sind
***	es weniger, wird piLength entsprechend verndert.
************************************************************************************/
bool RNSocket::receive(void* pvoidData, int* piLength, int iFlags/* =0 */)
{
	int iDataReceived;

	iDataReceived=recv(m_Socket, (char*)pvoidData, *piLength, iFlags);

	if (iDataReceived==SOCKET_ERROR)
		return false;

	*piLength=iDataReceived;

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::send
************************************************************************************/
/**	Sendet die Daten aus dem Buffer. Es wird versucht alle Daten zu senden, und nur 
*** bei einem Fehler wird abgebrochen.
************************************************************************************/
bool RNSocket::send(void* pvoidData, int* piLength, int iFlags/* =0 */)
{
	int iSendTotal = 0; // how many bytes weve sent
	int iBytesLeft = *piLength; // how many we have left to send
	int iBytesSend;

	while (iSendTotal < *piLength)
	{
		iBytesSend = ::send(m_Socket, 
							((char*)(pvoidData))+iSendTotal, 
							iBytesLeft, 
							iFlags);
		if (iBytesSend == SOCKET_ERROR )
		{
			*piLength = iSendTotal;
			return false;
		}

		iSendTotal += iBytesSend;
		iBytesLeft -= iBytesSend;
	}

	*piLength = iSendTotal;

	return true;
}

/************************************************************************************
**	Methode:			RNSocket::isReadable
************************************************************************************/
/**	berprft ob der Socket "readable" ist.
***
************************************************************************************/
bool RNSocket::isReadable(bool &bReadable, int iMilliSeconds)
{
	int iResult;

	fd_set fdSet;
	
	FD_ZERO(&fdSet);
	FD_SET(m_Socket, &fdSet);

	timeval tvTimeOut;

	tvTimeOut.tv_sec=iMilliSeconds/1000;
	tvTimeOut.tv_usec=(iMilliSeconds%1000)*1000;

	iResult = ::select(0, &fdSet, NULL, NULL, &tvTimeOut);

	// Fehler
	if(iResult == SOCKET_ERROR)
	{
		// konnte status nicht ermitteln
		bReadable = false;

		return false;
	}
	
	// Timeout
	if(iResult == 0)
	{
		bReadable = false;
		
		return true;
	}

	// hat geklappt - der socket ist in lesbarem zustand
	bReadable = true;

	return true;	
}

/************************************************************************************
**	Methode:			RNSocket::isWriteable
************************************************************************************/
/**	berprft ob der Socket "writeable" ist.
***
************************************************************************************/
bool RNSocket::isWriteable(bool &bWriteable, int iMilliSeconds)
{
	int iResult;

	fd_set fdSet;
	
	FD_ZERO(&fdSet);
	FD_SET(m_Socket, &fdSet);

	timeval tvTimeOut;

	tvTimeOut.tv_sec=iMilliSeconds/1000;
	tvTimeOut.tv_sec=(iMilliSeconds%1000)*1000;

	iResult = ::select(0, NULL, &fdSet, NULL, &tvTimeOut);

	// Fehler
	if(iResult == SOCKET_ERROR)
	{
		// konnte status nicht ermitteln		
		bWriteable = false;

		return false;
	}
	
	// Timeout
	if(iResult == 0)
	{
		bWriteable = false;

		return true;
	}

	// hat geklappt - der socket ist in schreibbarem zustand
	bWriteable = true;

	return true;
}

/************************************************************************************
**	Ende der Datei:		RNSocket.cpp
************************************************************************************/
